package org.noote.libs.bluetooth;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.UUID;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.util.Log;

public class SPPClient {

	private static final UUID _SPP_UUID = UUID
			.fromString("00001101-0000-1000-8000-00805F9B34FB"); // SPP
	private static final String TAG = "SPPClient";

	public static final int MESSAGE_CONNECTED = 0x00001001;
	public static final int MESSAGE_DECONNECTED = 0x00001002;
	public static final int MESSAGE_GETERROR = 0x00001100;
	public static final int MESSAGE_GETTEXT = 0x00002000;
	public static final int MESSAGE_GETCMD = 0x00004000;

	private Handler mHandler = null;
	private BluetoothAdapter mBluetoothAdapter = null;
	private BluetoothDevice mBluetoothDevice = null;
	private ConnectedThread mConnectedThread = null;
	
	private boolean _bIsConnected = false;
	
	public boolean isConnected(){return _bIsConnected;}

	public boolean connect(String sMACAdress, Handler h) {
		mHandler = h;

		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		if (mBluetoothAdapter == null) {
			_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
					"connect() : Bluetooth is not available.");
			Log.e(TAG, "Bluetooth is not available.");
			return false;
		}

		if (!mBluetoothAdapter.isEnabled()) {
			_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
					"connect() : Please enable your BT and re-run this program.");
			Log.e(TAG, "Please enable your BT and re-run this program.");
			return false;
		}

		BluetoothSocket mBluetoothSocket = null;
		mBluetoothDevice = mBluetoothAdapter.getRemoteDevice(sMACAdress);

		try {
			mBluetoothSocket = mBluetoothDevice
					.createRfcommSocketToServiceRecord(_SPP_UUID);
		} catch (IOException e) {
			_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
				"connect() : Socket creation failed.");	
			Log.e(TAG, "Socket creation failed.", e);
			return false;
		}

		mBluetoothAdapter.cancelDiscovery();

		try {
			mBluetoothSocket.connect();
			Log.e(TAG, "BT connection established, data transfer link open.");
		} catch (IOException e) {
			try {
				mBluetoothSocket.close();
			} catch (IOException e2) {
				Log.e(TAG, "Unable to close socket during connection failure",
						e2);
				_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
					"connect() : Unable to close socket during connection failure");				
			}
			return false;
		}

		(mConnectedThread = new ConnectedThread(mBluetoothSocket)).start();

		return true;
	}

	public boolean sendMessage(String sMessage) {
		if (mConnectedThread != null)
			return mConnectedThread.send(sMessage);

		Log.e(TAG, "sendMessage(): not connected");

		return false;
	}

	public boolean sendMessage(byte [] msg) {
		if (mConnectedThread != null)
			return mConnectedThread.send(msg);

		Log.e(TAG, "sendMessage(): not connected");

		return false;
	}

	private void _sendHandlerMessage(int iType, String sMessage) {
		if (mHandler == null) return;
		mHandler.sendMessage(mHandler.obtainMessage(iType, sMessage));
	}

	private class ConnectedThread extends Thread {
		private static final String TAG = "SPPClient.ConnectedThread";

		private final InputStream mmInStream;
		private final OutputStream mmOutStream;
		private final BluetoothSocket mSocket;

		public ConnectedThread(BluetoothSocket socket) {
			mSocket = socket;
			InputStream tmpIn = null;
			OutputStream tmpOut = null;

			// Get the input and output streams, using temp objects because
			// member streams are final
			try {
				tmpIn = socket.getInputStream();
				tmpOut = socket.getOutputStream();
			} catch (IOException e) {
				_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
					"ConnectedThread() : cannot get I/O handlers");
			}

			mmInStream = tmpIn;
			mmOutStream = tmpOut;
			try {
				mmOutStream.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		@Override
		public void run() {
			if(mmInStream == null || mmOutStream == null)
				return;
			
			_bIsConnected = true;
			try {
				mmOutStream.flush();
			} catch (IOException e) {
			}
			_sendHandlerMessage(SPPClient.MESSAGE_CONNECTED, null);
	    	
            BufferedReader in = new BufferedReader(new InputStreamReader(mmInStream));
	    	String line = null;
            try {
				while ((line = in.readLine()) != null) {
					Log.d(TAG, "Receive line : " + line);
					// have we a command or a result ?
					if(line.contains("#") && line.contains("$"))
					{
						int offset = 0;
						while(true)
						{
							int idx_start = line.indexOf("#", offset);
							if(idx_start == -1) break;
							int idx_end = line.indexOf("$", idx_start);
							if(idx_end == -1) break;
							offset = idx_end+1;
							
							_sendHandlerMessage(SPPClient.MESSAGE_GETCMD, line.substring(idx_start+1, idx_end));
						}
					}
					else // else it's only text...
					{
						_sendHandlerMessage(SPPClient.MESSAGE_GETTEXT, line);						
					}
				}
			} catch (IOException e) {
				_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
						"readLine() : I/O Exception " + e.toString());			
			}
			close();
			_bIsConnected = false;
	    }

		/* Call this from the main Activity to send data to the remote device */
		public boolean send(String sMessage) {
			PrintWriter out = new PrintWriter(new BufferedWriter(
					new OutputStreamWriter(mmOutStream)), true);
			out.println(sMessage);
			Log.d(TAG, "Sended message : " + sMessage);
			return true;
		}
		public boolean send(byte [] msg) {
			try {
				mmOutStream.write(msg);
				Log.d(TAG, "Sended byte : " + msg);				
				return true;
			} catch (IOException e) {
				_sendHandlerMessage(SPPClient.MESSAGE_GETERROR,
					"send(byte []) : I/O Exception " + e.toString());			
			}
			return false;
		}

		/* Call this from the main Activity to shutdown the connection */
		public void close() {
			try {
				mSocket.close();
			} catch (IOException e) {
			}
			_sendHandlerMessage(SPPClient.MESSAGE_DECONNECTED, null);
		}
	}
}
